/**
 * Copyright (C) 2012 Linutop SAS <http://www.linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "main.h"
#include "wk-pl-loop.h"
#include "wk-slideshow-item.h"

WkPlLoop *
wk_pl_loop_new ()
{
    WkPlLoop *loop;

    loop = g_new (WkPlLoop, 1);

    loop->priority = -1;
    loop->idle = FALSE;
    loop->data = NULL;

    return loop;
}

void
wk_pl_loop_free (WkPlLoop *loop)
{
    /* the data GSList items are WkSlideshow Items help in a GHashTable. They
     * are freed from the hash table, not from here. */
    g_slist_free (loop->data);
    g_free (loop);
}

static gboolean
wk_pl_loop_set_date (WkPlLoop *loop,
                     const gchar *start_date,
                     const gchar *end_date)
{
    loop->start_year = -1;
    loop->end_year = -1;

    if (!start_date || !end_date)
        return TRUE;

    if (3 != sscanf (start_date, "%d/%d/%d",
                     &loop->start_year, &loop->start_month, &loop->start_day))
    {
        loop->start_year = -1;
        return FALSE;
    }

    if (3 != sscanf (end_date, "%d/%d/%d",
                     &loop->end_year, &loop->end_month, &loop->end_day))
    {
        loop->end_year = -1;
        return FALSE;
    }

    if (strcmp (start_date, end_date) > 0)
        return FALSE;

    /* make sure to have a correct format in the strings */
    snprintf (loop->start_date, 11, "%04d/%02d/%02d",
              loop->start_year, loop->start_month, loop->start_day);
    snprintf (loop->end_date, 11, "%04d/%02d/%02d",
              loop->end_year, loop->end_month, loop->end_day);

    return TRUE;
}

static gboolean
wk_pl_loop_set_time (WkPlLoop *loop,
                     const gchar *start_time,
                     const gchar *end_time,
                     const gchar *weekdays)
{
    char *p;

    loop->start_hour = -1;
    loop->end_hour = -1;
    loop->weekdays = 0;
    loop->start_time[0] = 0;

    if (weekdays)
    {
        loop->weekdays = 0;
        for (p=(char *)weekdays;*p;p++)
        {
            if (*p >= '0' && *p <= '6')
                loop->weekdays |= 1 << (*p - '0');
        }
    }

    if (!start_time)
        return TRUE;

    if (2 != sscanf (start_time, "%d:%d", &loop->start_hour, &loop->start_min))
    {
        loop->start_hour = -1;
        return FALSE;
    }

    /* if no end defined, or badly defined, end is 23:59 */
    if (2 != sscanf (end_time, "%d:%d", &loop->end_hour, &loop->end_min))
    {
        loop->end_hour = 23;
        loop->end_min = 59;
    }

    /* same if end is 00:00 */
    if ((loop->end_hour == 0) && (loop->end_min == 0))
    {
        loop->end_hour = 23;
        loop->end_min = 59;
    }

    /* if end after start? */
    if ((loop->end_hour * 60 + loop->end_min)
            <= (loop->start_hour * 60 + loop->start_min))
    {
        return FALSE;
    }

    /* make sure to have a correct format in the strings */
    snprintf (loop->start_time, 6, "%02d:%02d",
              loop->start_hour, loop->start_min);
    snprintf (loop->end_time, 6, "%02d:%02d",
              loop->end_hour, loop->end_min);

    return TRUE;
}

void
wk_pl_loop_set_idle (WkPlLoop *loop, gboolean idle)
{
    loop->idle = idle;
    /* TODO: create fake playlist */
}

gboolean
wk_pl_loop_set_from_markup (WkPlLoop *loop,
                            int id,
                            const gchar *start_time,
                            const gchar *end_time,
                            const gchar *weekdays,
                            const gchar *start_date,
                            const gchar *end_date,
                            gboolean idle)
{
    loop->id = id;

    if (!wk_pl_loop_set_date (loop, start_date, end_date))
        return FALSE;

    if (!wk_pl_loop_set_time (loop, start_time, end_time, weekdays))
        return FALSE;

    wk_pl_loop_set_idle (loop, idle);

    /* prio level */
    loop->priority = 0;
    loop->priority += (loop->weekdays > 0)    ? 1 : 0;
    loop->priority += (loop->start_hour >= 0) ? 1 : 0;
    loop->priority += (loop->start_year >= 0) ? 1 : 0;

    return TRUE;
}

gboolean
wk_pl_loop_load_from_xml_data (WkPlLoop *loop,
                               const gchar **attribute_names,
                               const gchar **attribute_values)
{
    const gchar *start_date = NULL;;
    const gchar *end_date = NULL;
    const gchar *start_time = NULL;
    const gchar *end_time = NULL;
    const gchar *weekdays = NULL;
    gboolean idle = FALSE;
    int i, id = -1;

    for (i=0; attribute_names[i] != NULL; i++)
    {
        if ((0 == strcmp (attribute_names[i], "start"))
                && (attribute_values[i][0] != 0))
        {
            start_time = attribute_values[i];
        }
        else if ((0 == strcmp (attribute_names[i], "end"))
                && (attribute_values[i][0] != 0))
        {
            end_time = attribute_values[i];
        }
        else if ((0 == strcmp (attribute_names[i], "weekdays"))
                && (attribute_values[i][0] != 0))
        {
            weekdays = attribute_values[i];
        }
        else if  ((0 == strcmp (attribute_names[i], "from"))
                && (attribute_values[i][0] != 0))
        {
            start_date = attribute_values[i];
        }
        else if  ((0 == strcmp (attribute_names[i], "to"))
                && (attribute_values[i][0] != 0))
        {
            end_date = attribute_values[i];
        }
        else if (0 == strcmp (attribute_names[i], "id"))
        {
            id = atoi (attribute_values[i]);
        }
        else if (0 == strcmp (attribute_names[i], "idle"))
        {
            idle = strcmp (attribute_values[i], "1") ? FALSE : TRUE;
        }
    }

    if (id == -1)
        return FALSE;

    return wk_pl_loop_set_from_markup (loop, id,
                                       start_time, end_time, weekdays,
                                       start_date, end_date,
                                       idle);
}

gboolean
wk_pl_loop_fits_now (WkPlLoop *loop)
{
    time_t t_now;
    struct tm *now;
    gchar now_date[11];
    gchar now_time[6];
    gboolean ret = TRUE;

    t_now = time (NULL);
    now = localtime (&t_now);
    strftime (now_date, 11, "%Y/%m/%d", now);
    strftime (now_time, 6, "%H:%M", now);

    DEBUG ("Loop: %d", loop->id);
    DEBUG ("Now: %s %s", now_date, now_time);

    /* check dates */
    if (loop->start_year >= 0)
    {
        if ((strcmp (loop->start_date, now_date) > 0)
            || (strcmp (now_date, loop->end_date) > 0))
        {
            DEBUG ("Date not matching");
            ret = FALSE;
        }

    }

    /* check day of week */
    if (loop->weekdays > 0 )
    {
        if ((loop->weekdays & (1 << now->tm_wday)) == 0)
        {
            DEBUG ("Week day not matching");
            ret = FALSE;
        }
    }

    /* check the time */
    if (loop->start_hour >= 0)
    {
        if ((strcmp (loop->start_time, now_time) > 0)
            || (strcmp (now_time, loop->end_time) > 0))
        {
            DEBUG ("%s %s %s", loop->start_time, now_time, loop->end_time);
            DEBUG ("Time not matching");
            ret = FALSE;
        }
    }

    return ret;
}

#ifdef WITH_DEBUG
static void
print_data (gpointer data, gpointer user_data)
{
    WkSlideshowItem *item = (WkSlideshowItem *) data;

    if (data)
        printf ("  data %3d - %s (%ds - %d)\n", item->id, item->uri, item->timeout, item->type);
}

void
wk_pl_loop_print_data (WkPlLoop *loop)
{
    g_slist_foreach (loop->data, print_data, NULL);
}

void wk_pl_loop_print (WkPlLoop *loop)
{
    printf ("Loop %d (idle=%s)\n", loop->id, loop->idle ? "Y" : "N");
    wk_pl_loop_print_data (loop);
}
#endif

