/**
 * Copyright (C) 2012 Linutop SAS <http://www.linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>

#include <glib.h>
#include <glib/gstdio.h>

#include "text-utils.h"
#include "wk-download.h"
#include "wk-pl.h"

struct selection {
    gint priority;
    WkPlLoop *loop;
};

static gboolean in_loop = FALSE;

static void wk_pl_add_idle_item (WkPl *pl);
static void clear_cache (GList *tokeep,
                         const gchar *data_dir,
                         const gchar *etag_dir);

WkPl *
wk_pl_new ()
{
    WkPl *pl;

    pl = g_new (WkPl, 1);

    pl->loops = g_hash_table_new_full (g_int_hash, g_int_equal, NULL, (GDestroyNotify) wk_pl_loop_free);
    pl->data = g_hash_table_new_full (g_int_hash, g_int_equal, NULL, (GDestroyNotify) wk_slideshow_item_free);
    wk_pl_add_idle_item (pl);
    pl->selected_loop = NULL;
    pl->current_id = 1;

    return pl;
}

void
wk_pl_add_loop (WkPl *pl, WkPlLoop *loop)
{
    if (loop->id == 0)
        pl->selected_loop = loop;

    g_hash_table_insert (pl->loops, &(loop->id), loop);
}

void
wk_pl_add_data (WkPl *pl, WkSlideshowItem *item)
{
    g_hash_table_insert (pl->data, &item->id, item);
    DEBUG ("Adding data: %s", item->uri);
    DEBUG ("Number of items: %d", g_hash_table_size (pl->data));
}

void
wk_pl_reset (WkPl *pl)
{
    g_hash_table_remove_all (pl->loops);
    g_hash_table_remove_all (pl->data);
    wk_pl_add_idle_item (pl);
    pl->selected_loop = NULL;
    pl->current_id = 1;
}

void
wk_pl_free (WkPl *pl)
{
    g_hash_table_destroy (pl->loops);
    g_hash_table_destroy (pl->data);
    g_free (pl);
}

static void
wk_pl_add_idle_item (WkPl * pl)
{
    WkSlideshowItem *item = wk_slideshow_item_new_idle ();
    wk_pl_add_data (pl, item);
}

static inline WkSlideshowItem *
wk_pl_get_idle_item (WkPl *pl)
{
    int id = 0;

    return g_hash_table_lookup (pl->data, &id);
}

void
wk_pl_set_loop_idle (WkPl *pl, WkPlLoop *loop)
{
    wk_pl_loop_add_data (loop, wk_pl_get_idle_item (pl));
}

static void
end_element (GMarkupParseContext *context,
             const gchar         *element_name,
             gpointer             user_data,
             GError             **error)
{
    if (in_loop && (0 == strcmp (element_name, "loop")))
    {
        in_loop = FALSE;
    }
}

static void
start_element (GMarkupParseContext *context,
               const gchar         *element_name,
               const gchar        **attribute_names,
               const gchar        **attribute_values,
               gpointer             user_data,
               GError             **error)
{
    WkPl *pl = (WkPl *) user_data;
    static WkPlLoop *current_loop = NULL;

    /* items from a loop */
    if (in_loop && (current_loop->idle == FALSE))
    {
        if (strcmp (element_name, "item") != 0)
            return;

        WkSlideshowItem *item;

        item = wk_slideshow_item_new_from_xml_data (pl->current_id,
                                                    attribute_names,
                                                    attribute_values);
        if (!item)
            return;

        pl->current_id++;

        wk_pl_add_data (pl, item);
        wk_pl_loop_add_data (current_loop, item);
    }

    if (strcmp (element_name, "loop") == 0)
    {
        WkPlLoop *loop = wk_pl_loop_new ();
        if (!wk_pl_loop_load_from_xml_data (loop, attribute_names, attribute_values))
            return;

        wk_pl_add_loop (pl, loop);
        current_loop = loop;

        if (current_loop->idle == TRUE)
            wk_pl_set_loop_idle (pl, current_loop);

        in_loop = TRUE;
    }
}

static const GMarkupParser parser = {
    start_element,
    end_element,
    NULL,
    NULL,
    NULL
};

gboolean
wk_pl_set_from_xml (WkPl *pl, const gchar *xml)
{
    GMarkupParseContext *context;
    gboolean ret = TRUE;

    wk_pl_reset (pl);

    context = g_markup_parse_context_new (&parser, 0, pl, NULL);
    if (!g_markup_parse_context_parse (context, xml, strlen (xml), NULL))
    {
        ret = FALSE;
        goto out;
    }

out:
    g_markup_parse_context_free (context);
    return ret;
}

static void
select_pl_loop (int *id, WkPlLoop *loop, struct selection *sel)
{
    if (wk_pl_loop_fits_now (loop)
        && (loop->priority > sel->priority))
    {
        sel->priority = loop->priority;
        sel->loop = loop;
    }
}

void
wk_pl_select (WkPl *pl)
{
    struct selection selection = { 0, NULL };
    int i = 0;

    pl->selected_loop = g_hash_table_lookup (pl->loops, &i);
    g_hash_table_foreach (pl->loops, (GHFunc) select_pl_loop, &selection);
    if (selection.loop)
        pl->selected_loop = selection.loop;
}

static int
dir_filter (const struct dirent *d)
{
    return d->d_name[0] != '.';
}

static void
append_dir_to_list (const gchar *dirname, gint t, WkPl *pl, WkPlLoop *loop)
{
    struct dirent **names;
    gchar *uri = NULL;
    gchar *filename = NULL;
    int n, i;
    WkSlideshowItem *item;

    n = scandir (dirname, &names, dir_filter, alphasort);

    if (n < 0)
        return;

    for (i=0; i<n; i++)
    {
        uri = g_strdup_printf ("file://%s/%s", dirname, names[i]->d_name);
        free (names[i]);
        filename = uri+7;

        if (g_file_test (filename, G_FILE_TEST_IS_DIR))
        {
            append_dir_to_list (filename, t, pl, loop);
        }
        else
        {
            item = wk_slideshow_item_new (pl->current_id, uri, 0, t, FALSE);
            wk_pl_add_data (pl, item);
            wk_pl_loop_add_data (loop, item);
            pl->current_id++;
        }
        g_free (uri);
    }

    free (names);
}

gboolean
wk_pl_set_from_list (WkPl *pl, const gchar *str)
{
    WkSlideshowItem *item;
    GRegex *regex;
    gchar **lines = NULL;
    gchar **p;
    int t;
    WkPlLoop *loop;
    gboolean ret = TRUE;

    wk_pl_reset (pl);

    regex = g_regex_new ("^(.*) ([0-9]+)$", 0, 0, NULL);
    lines = g_strsplit (str, "\n", 0);
    if (g_strv_length (lines) == 0)
    {
        ret = FALSE;
        goto out;
    }

    loop = wk_pl_loop_new ();
    loop->id = 0;
    wk_pl_add_loop (pl, loop);
    pl->selected_loop = loop;

    for (p=lines+1; *p; p++)
    {
        char **tmp = NULL;

        tmp = g_regex_split (regex, *p, 0);
        if (g_strv_length (tmp) == 4)
        {
            t = atoi (tmp[2]);
            if (g_file_test (tmp[1]+7, G_FILE_TEST_IS_DIR))
            {
                append_dir_to_list (tmp[1]+7, t, pl, loop);
                continue;
            }
            item = wk_slideshow_item_new (pl->current_id, tmp[1], 0, t, FALSE);
            wk_pl_add_data (pl, item);
            wk_pl_loop_add_data (loop, item);
            pl->current_id++;
        }
        g_strfreev (tmp);
    }

out:
    g_strfreev (lines);
    g_regex_unref (regex);

    return ret;
}

static const gchar *todownloadprefixes[] = {
    "http://",
    "ftp://",
    "https://",
    NULL
};

static gboolean
to_download_prefix (const gchar *uri)
{
    gboolean ret = FALSE;
    const gchar **p;

    for (p=todownloadprefixes;*p;p++)
    {
        if (g_str_has_prefix (uri, *p))
        {
            ret = TRUE;
            break;
        }
    }

    return ret;
}

static gboolean
item_needs_download (WkSlideshowItem *item)
{
    if (item->type == WK_SLIDESHOW_ITEM_TYPE_IDLE)
        return FALSE;

    if (item->type == WK_SLIDESHOW_ITEM_TYPE_WEB)
        return FALSE;

    if (!to_download_prefix (item->uri))
        return FALSE;

    return TRUE;
}

void
wk_pl_set_cache (WkPl *pl, const gchar *data_dir, const gchar *etag_dir)
{
    GList *p;
    gboolean success;
    gchar *tmp;
    gchar *target;
    gchar *etag_file;
    GList *tokeep = NULL;

    DEBUG ("start");

    /* Get a list of what needs to be downloaded... */
    for (p = g_hash_table_get_values (pl->data); p; p = p->next)
    {
        if (item_needs_download (p->data))
            tokeep = g_list_append (tokeep, p->data);
    }

    /* ... to remove old data first */
    clear_cache (tokeep, data_dir, etag_dir);

    /* Then perform the downloads */
    for (p = tokeep; p; p = p->next)
    {
        WkSlideshowItem *item = (WkSlideshowItem *) p->data;

        tmp = g_path_get_basename (item->uri);
        target = g_strconcat (data_dir, tmp, NULL);
        etag_file = g_strconcat (etag_dir, tmp, NULL);
        g_free (tmp);

        if (!g_file_test (target, G_FILE_TEST_EXISTS))
        {
            success = wk_download_perform (item->uri, target, etag_file);
            DEBUG ("downloaded: %s => %s (%d)", item->uri, target, success);
        }
        else if (wk_download_is_needed (item->uri, target, etag_file))
        {
            success = wk_download_perform (item->uri, target, etag_file);
            DEBUG ("re-downloaded: %s => %s (%d)", item->uri, target, success);
        }
        else
            success = TRUE;

        g_free (etag_file);

        if (success)
        {
            g_free (item->uri);
            if (item->type == WK_SLIDESHOW_ITEM_TYPE_DOCUMENT)
            {
                /* update the text with local data */
                text_fill_cache (target, data_dir);
                item->uri = g_strconcat ("file://", target, NULL);
                g_free (target);
            }
            else
            {
                item->uri = target;
            }
        } else {
            item->type = WK_SLIDESHOW_ITEM_TYPE_WEB;
        }
    }

    g_list_free (tokeep);

    DEBUG ("end");
}

static gint
tokeep_compare_func (WkSlideshowItem *item,
                     const gchar *filename)
{
    gchar *tmp;
    gint ret;

    tmp = g_path_get_basename (item->uri);
    ret = strcmp (filename, tmp);

    g_free (tmp);

    return ret;
}

void
static clear_cache (GList *tokeep,
                    const gchar *data_dir,
                    const gchar *etag_dir)
{
    GDir *dir;
    const gchar *f;
    gchar *data_path, *etag_path;

    dir = g_dir_open (data_dir, 0, NULL);
    if (!dir)
        return;

    while ((f = g_dir_read_name (dir)) != NULL)
    {
        data_path = g_build_filename (data_dir, f, NULL);
        DEBUG ("Testing drop of cached file: %s", data_path);

        if (g_list_find_custom (tokeep, f,
                    (GCompareFunc) tokeep_compare_func))
        {
            DEBUG ("NOT deleting");
            g_free (data_path);
            continue;
        }

        DEBUG ("Deleting");

        etag_path = g_build_filename (etag_dir, f, NULL);

        g_unlink (data_path);
        g_unlink (etag_path);

        g_free (data_path);
        g_free (etag_path);
    }

    g_dir_close (dir);
}

#ifdef WITH_DEBUG
static void
print_loop (gpointer key, gpointer value, gpointer data)
{
    wk_pl_loop_print (value);
}

static void
print_data (gpointer key, gpointer value, gpointer data)
{
    WkSlideshowItem *item = (WkSlideshowItem *) value;

    printf ("Data id %3d - %s (timing %d)\n", item->id, item->uri, item->timeout);
}

void
wk_pl_print (WkPl *pl)
{
    g_hash_table_foreach (pl->loops, print_loop, NULL);
    g_hash_table_foreach (pl->data, print_data, NULL);
    if (pl->selected_loop)
        printf ("Selected: %d\n", pl->selected_loop->id);
    else
        printf ("Selected: failed\n");
}
#endif

