/**
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <glib/gstdio.h>

#include <gdk/gdkkeysyms.h>

#include "settings.h"
#include "gtk-extensions.h"

#include "xfce_framebox.h"
#include "config-kiosk.h"

#include "utils.h"

WkConfigItemDesc kiosk_descs[] =
{
    { "main", "can_end_session", WK_CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "can_access_fs", WK_CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "with_clear", WK_CONFIG_ITEM_TYPE_BOOLEAN, "true" },
    { "main", "default_url", WK_CONFIG_ITEM_TYPE_STRING, "http://linutop.com" },
    { "main", "max_history", WK_CONFIG_ITEM_TYPE_INT, "50" },
    { "main", "max_windows", WK_CONFIG_ITEM_TYPE_INT, "0" },
    { "main", "stop_session_after_seconds", WK_CONFIG_ITEM_TYPE_INT, "120" },

    { "main", "with_timer", WK_CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "default_timer", WK_CONFIG_ITEM_TYPE_INT, "900" },

    { "main", "start_with_session", WK_CONFIG_ITEM_TYPE_BOOLEAN, "true" },
    { "main", "hide_toolbar", WK_CONFIG_ITEM_TYPE_BOOLEAN, "false" },
    { "main", "post_session_uri", WK_CONFIG_ITEM_TYPE_STRING, "" },
    { "main", "post_session_uri_time", WK_CONFIG_ITEM_TYPE_INT, "" },
    /* { "main", "prestart", WK_CONFIG_ITEM_TYPE_STRING, "" }, */

    { "main", "list_mode", WK_CONFIG_ITEM_TYPE_STRING, "none" },
    { "main", "list_use_regex", WK_CONFIG_ITEM_TYPE_BOOLEAN, "false" },

    { NULL }
};

const gchar *combo_list_types[3][2] =
{
    { "none", N_("None") },
    { "whitelist", N_("Whitelist") },
    { "blacklist", N_("Blacklist") },
};

const gchar *combo_after_session[3][2] =
{
    { "homepage", N_("Show the homepage") },
    { "slideshow", N_("Start a slideshow") },
    { "session", N_("Re-start a session") },
};

static GHashTable *hash;
static GtkListStore *list_store, *type_store, *after_store;
static GtkWidget *combo;
static GtkWidget *treeview;
static GtkWidget *add_btn, *rm_btn;
static GtkWidget *advanced_exp;
static GtkWidget *stop_session_label;
static GtkWidget *session_at_startup_chk = NULL;
static WkKeyFile *kfp;

static void save_list (const gchar *filename);

inline GHashTable *
config_kiosk_get_hash (WkKeyFile *kf)
{
    kfp = kf;
    hash = handled_settings_get_from_array (kfp, kiosk_descs);
    return hash;
}

inline void
kiosk_set_session_at_startup (gboolean at_startup)
{
    if (session_at_startup_chk)
    {
        gtk_toggle_button_set_active (
                GTK_TOGGLE_BUTTON (session_at_startup_chk),
                at_startup);
    }
}

inline void
kiosk_set_timeout_label (gboolean with_display)
{
    gtk_label_set_text (GTK_LABEL (stop_session_label),
            with_display ? _("Start the screen saver after:")
                         : _("Restart a web session after:"));
}

static void
populate_list (const gchar *filename)
{
    const gchar *listfile;
    gchar *contents;
    gchar **lines;
    gchar **linep;
    GtkTreeIter iter;

    if (filename)
        listfile = filename;
    else
        listfile = wk_key_file_get_filename (kfp, WK_KEY_FILE_FILENAME_LIST);

    if (! g_file_get_contents (listfile, &contents, NULL, NULL))
        return;

    gtk_list_store_clear (list_store);

    lines = g_strsplit (contents, "\n", -1);
    for (linep = lines; *linep; linep++)
    {
        if (strlen (*linep) == 0)
            continue;

        gtk_list_store_append (list_store, &iter);
        gtk_list_store_set (list_store, &iter, 0, *linep, -1);
    }
    g_strfreev (lines);
    g_free (contents);
}

static void
populate_stores ()
{
    WkConfigItem *item_mode;
    gchar *mode;
    GtkTreeIter iter;
    int i;
    gboolean valid = FALSE;

    /* combo box for list type */
    item_mode = g_hash_table_lookup (hash, "main:list_mode");
    mode = (gchar *) wk_config_item_get_value (item_mode);
    for (i=0;i<3;i++)
    {
        gtk_list_store_append (type_store, &iter);
        gtk_list_store_set (type_store, &iter, 0, combo_list_types[i][0],
                                               1, _(combo_list_types[i][1]),
                                               -1);
        if (strcmp (mode, combo_list_types[i][0]) == 0)
        {
            gtk_combo_box_set_active (GTK_COMBO_BOX (combo), i);
            valid = TRUE;
        }
    }
    if (! valid)
        gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);

    populate_list (NULL);
}

static void
on_treeview_changed (GtkTreeView *treeview, gpointer data)
{
    if (gtk_tree_selection_count_selected_rows (
            gtk_tree_view_get_selection (treeview)))
        gtk_widget_set_sensitive (rm_btn, TRUE);
    else
        gtk_widget_set_sensitive (rm_btn, FALSE);
}

static void
on_save_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkWidget *d;
    char *filename;

    d = gtk_file_chooser_dialog_new (_("Save the list"),
                                     GTK_WINDOW (main_window),
                                     GTK_FILE_CHOOSER_ACTION_SAVE,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                     NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (d), TRUE);

    if (gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (d));
        save_list (filename);
        g_free (filename);
    }
    gtk_widget_destroy (d);

}

static void
on_open_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkWidget *d;
    char *filename;

    d = gtk_file_chooser_dialog_new (_("Open a list"),
                                     GTK_WINDOW (main_window),
                                     GTK_FILE_CHOOSER_ACTION_OPEN,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                     NULL);
    gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (d), TRUE);

    if (gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_ACCEPT)
    {
        filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (d));
        populate_list (filename);
        g_free (filename);
    }
    gtk_widget_destroy (d);
}
static void
on_add_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkWidget *d, *add_entry, *label;
    const gchar *text;
    GtkTreeIter iter;

    d = gtk_dialog_new_with_buttons (_("Add a URL"),
                                     GTK_WINDOW (main_window),
                                     GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     GTK_STOCK_ADD, GTK_RESPONSE_OK,
                                     NULL);
    gtk_dialog_set_default_response (GTK_DIALOG (d), GTK_RESPONSE_OK);
    gtk_container_set_border_width (GTK_CONTAINER (d), 4);

    label = gtk_label_new (_("Type the URL scheme that you want to add:\n"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), label, FALSE, FALSE, 2);
    gtk_widget_show (label);

    add_entry = gtk_entry_new ();
    gtk_entry_set_text (GTK_ENTRY (add_entry), "http://");
    gtk_entry_set_activates_default (GTK_ENTRY (add_entry), TRUE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), add_entry, TRUE, TRUE, 2);
    gtk_widget_show (add_entry);

    if (gtk_dialog_run (GTK_DIALOG (d)) != GTK_RESPONSE_OK)
        goto out;

    text = gtk_entry_get_text (GTK_ENTRY (add_entry));
    if (strlen (text) < 1)
        goto out;

    gtk_list_store_append (list_store, &iter);
    gtk_list_store_set (list_store, &iter, 0, g_strdup (text), -1);

out:
    gtk_widget_destroy (d);
    return;
}

static void
on_rm_btn_clicked (GtkButton *btn, gpointer data)
{
    GtkTreeModel *model = GTK_TREE_MODEL (list_store);
    GList *list = NULL;
    GList *p;

    list = gtk_tree_selection_get_selected_rows (
        gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview)), &model);

    /* loop on the list !*/
    for (p=list; p && p->data; p=p->next)
    {
        GtkTreePath *path = (GtkTreePath *) p->data;
        p->data = gtk_tree_row_reference_new (model, path);
        gtk_tree_path_free (path);
    }

    for (p=list; p && p->data; p=p->next)
    {
        GtkTreeIter iter;
        GtkTreePath *path = gtk_tree_row_reference_get_path (p->data);

        if (path && gtk_tree_model_get_iter (model, &iter, path))
        {
            gtk_list_store_remove (list_store, &iter);
            gtk_tree_row_reference_free (p->data);
            gtk_tree_path_free (path);
        }
    }
    g_list_free (list);

    on_treeview_changed (GTK_TREE_VIEW (treeview), NULL);
}

static void
on_combo_changed (GtkComboBox *combo, WkConfigItem *item)
{
    GtkTreeIter iter;
    gchar *mode = NULL;

    gtk_combo_box_get_active_iter (combo, &iter);
    gtk_tree_model_get (GTK_TREE_MODEL (type_store), &iter, 0, &mode, -1);

    /* mode will be freed with the WkConfigItem */
    wk_config_item_set_value (item, mode, TRUE);
}

static void
on_key_press (GtkWidget *w, GdkEvent *event, gpointer data)
{
    GdkEventKey *ek = (GdkEventKey *) event;

    if (ek->keyval == GDK_Delete)
        gtk_widget_activate (GTK_WIDGET (data));
}

GtkWidget *
config_kiosk_get_widget (GHashTable *hash)
{
    GtkWidget *mvbox; /* returned widget */
    GtkWidget *label, *frame, *box, *box2, *box3, *hbox, *cvbox;
    GtkWidget *btn;
    GtkWidget *sep;
    GtkWidget *open_btn, *save_btn;
    GtkWidget *scw;
    GtkSizeGroup *size_group;
    GtkCellRenderer *renderer;
    GtkTreeViewColumn *column;

    mvbox = gtk_vbox_new (FALSE, 2);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mvbox), hbox, FALSE, FALSE, 0);

    cvbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), cvbox, FALSE, FALSE, 0);

    frame = xfce_framebox_new (_("Interface"), TRUE);
    box = gtk_vbox_new (FALSE, 2);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_box_pack_start (GTK_BOX (cvbox), frame, FALSE, FALSE, 6);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Hide the tool bar"),
                            g_hash_table_lookup (hash, "main:hide_toolbar"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("User can end his web session"),
                            g_hash_table_lookup (hash, "main:can_end_session"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("User can browse the file system"),
                            g_hash_table_lookup (hash, "main:can_access_fs"),
                            FALSE),
                        FALSE, FALSE, 0);

    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Display the 'clear' button"),
                            g_hash_table_lookup (hash, "main:with_clear"),
                            FALSE),
                        FALSE, FALSE, 0);

    session_at_startup_chk = wk_gtk_check_button_new (
                                _("Begin a web session at startup"),
                                g_hash_table_lookup (hash, "main:start_with_session"),
                                FALSE);
    gtk_box_pack_start (GTK_BOX (box), session_at_startup_chk, FALSE, FALSE, 0);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Home page: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_entry_new (
                            g_hash_table_lookup (hash, "main:default_url")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 0);

    stop_session_label = gtk_label_new (_("Restart a web session after:"));
    gtk_misc_set_alignment (GTK_MISC (stop_session_label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box), stop_session_label, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 3600, 10,
                            g_hash_table_lookup (hash, "main:stop_session_after_seconds")),
                        FALSE, FALSE, 2);
    label = gtk_label_new (_("seconds of inactivity"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, FALSE, FALSE, 0);

    /* restrictions */
    cvbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), cvbox, TRUE, TRUE, 0);

    frame = xfce_framebox_new (_("Restrictions (whitelist/blacklist)"), TRUE);
    box = gtk_vbox_new (FALSE, 2);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_box_pack_start (GTK_BOX (cvbox), frame, FALSE, FALSE, 6);

    type_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
    list_store = gtk_list_store_new (1, G_TYPE_STRING);
    after_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);

    combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (type_store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo), renderer, "text", 1);
    g_signal_connect (G_OBJECT (combo), "changed",
                      G_CALLBACK (on_combo_changed),
                      g_hash_table_lookup (hash, "main:list_mode"));

    box2 = gtk_hbox_new (FALSE, 6);

    scw = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scw),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);

    treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (list_store));
    gtk_tree_selection_set_mode (gtk_tree_view_get_selection (
                                    GTK_TREE_VIEW (treeview)),
                                 GTK_SELECTION_MULTIPLE);
    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes (
        "",
        renderer,
        "text", 0,
        NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);

    box3 = gtk_vbox_new (FALSE, 2);

    add_btn = gtk_button_new_from_stock (GTK_STOCK_ADD);
    rm_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
    gtk_widget_set_sensitive (rm_btn, FALSE);

    sep = gtk_hseparator_new ();
    open_btn = gtk_button_new_from_stock (GTK_STOCK_OPEN);
    save_btn = gtk_button_new_from_stock (GTK_STOCK_SAVE_AS);

    gtk_box_pack_start (GTK_BOX (box), combo, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box),
                        wk_gtk_check_button_new (_("Use regular expressions"),
                            g_hash_table_lookup (hash, "main:list_use_regex"),
                            FALSE),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box2, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (box2), scw, TRUE, TRUE, 2);
    gtk_container_add (GTK_CONTAINER (scw), treeview);
    gtk_box_pack_start (GTK_BOX (box2), box3, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), add_btn, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box3), rm_btn, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box3), sep, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box3), open_btn, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box3), save_btn, FALSE, FALSE, 0);

    g_signal_connect (G_OBJECT (treeview), "key-press-event",
                      G_CALLBACK (on_key_press), rm_btn);
    g_signal_connect (G_OBJECT (treeview), "cursor-changed",
                      G_CALLBACK (on_treeview_changed), NULL);
    g_signal_connect (G_OBJECT (add_btn), "clicked",
                      G_CALLBACK (on_add_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (rm_btn), "clicked",
                      G_CALLBACK (on_rm_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (open_btn), "clicked",
                      G_CALLBACK (on_open_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (save_btn), "clicked",
                      G_CALLBACK (on_save_btn_clicked), NULL);

    /* advanced settings */
    advanced_exp = gtk_expander_new (_("<b>Advanced settings</b>"));
    gtk_expander_set_use_markup (GTK_EXPANDER (advanced_exp), TRUE);
    box = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (mvbox), advanced_exp, FALSE, FALSE, 0);
    frame = xfce_framebox_new (NULL, TRUE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), box);
    gtk_container_add (GTK_CONTAINER (advanced_exp), frame);

    box3 = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box3, FALSE, FALSE, 4);

    size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Max items in history: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 100, 1,
                            g_hash_table_lookup (hash, "main:max_history")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), box2, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Max opened windows: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_size_group_add_widget (size_group, label);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 50, 1,
                            g_hash_table_lookup (hash, "main:max_windows")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), box2, FALSE, FALSE, 2);

    sep = gtk_vseparator_new ();
    gtk_box_pack_start (GTK_BOX (box), sep, FALSE, FALSE, 2);

    box3 = gtk_vbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box), box3, FALSE, FALSE, 4);

    box2 = gtk_hbox_new (FALSE, 2);
    btn = wk_gtk_check_button_new (_("Use a session timer"),
                                   g_hash_table_lookup (hash, "main:with_timer"),
                                   FALSE);
    gtk_box_pack_start (GTK_BOX (box2), btn, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (1, 500000, 1,
                            g_hash_table_lookup (hash, "main:default_timer")),
                        FALSE, FALSE, 2);
    label = gtk_label_new (_("secondes by default"));
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), box2, FALSE, FALSE, 2);

    box2 = gtk_hbox_new (FALSE, 2);
    label = gtk_label_new (_("Post session URL: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_entry_new (
                            g_hash_table_lookup (hash, "main:post_session_uri")),
                        FALSE, FALSE, 2);
    label = gtk_label_new (_(" shown "));
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2),
                        wk_gtk_spin_button_new_with_range (0, 100, 1,
                            g_hash_table_lookup (hash, "main:post_session_uri_time")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box2), gtk_label_new (_("second(s)")),
                        FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (box3), box2, FALSE, FALSE, 2);

    populate_stores ();

    gtk_widget_show_all (mvbox);

    return mvbox;
}

static gboolean
append_data (GtkTreeModel *model, GtkTreePath *path,
             GtkTreeIter *iter, gpointer data)
{
    gchar *line = NULL;
    gchar *tmp = NULL;
    gchar **datap = (gchar **) data;

    gtk_tree_model_get (model, iter, 0, &line, -1);

    if (*datap == NULL)
        *datap = g_strdup (line);
    else
    {
        tmp = g_strconcat (*datap, "\n", line, NULL);
        g_free (*datap);
        *datap = tmp;
    }

    g_free (line);

    return FALSE;
}

static void
save_list (const gchar *filename)
{
    gchar *tmpname = NULL;
    gchar *data = NULL;

    tmpname = g_strdup_printf ("%s.tmp", filename);

    gtk_tree_model_foreach (GTK_TREE_MODEL (list_store), append_data, &data);
    if (!data)
        data = g_strdup ("");

    if (!create_dir_for_file (tmpname)
        || !g_file_set_contents (tmpname, data, -1, NULL)
        || (g_rename (tmpname, filename) != 0))
    {
        gtk_dialog_run (GTK_DIALOG (gtk_message_dialog_new (
            GTK_WINDOW (main_window),
            0,
            GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
            _("Unable to save the listing file "
            "to\n%s"), filename)));
    }

    g_free (tmpname);
    g_free (data);
}

void
config_kiosk_save_config (WkKeyFile *kf, GHashTable *hash)
{
    const gchar *filename;

    handled_settings_set_from_hash (kf, hash);

    filename = wk_key_file_get_filename (kfp, WK_KEY_FILE_FILENAME_LIST);
    save_list (filename);
}
